function [TransMatrix, cholCov] = Tintspade(TscPow,GTtscPow,scale,cholG,h,q,pcoef,qfactor,qrank)
% Returns the discrete transition matrix TransMatrix and Choleski factor cholCov of the
% discrete transition error variance matrix for the integral over the interval h of the
% continuous time system with rate transition matrix T and Choleski factor cholG of the
% variance matrix of the Brownian disturbance.
% Scaled powers of T are supplied in TScPow and products of cholG with scaled powers of
% T' in GTtscPow. These are generated from T and cholG using function powscalT.m and
% need only be formed once together with the scaling factor scale for application of
% Tintspade with different values of h. A Pade approximation method is used which
% requires the quantities pcoef, qfactor and qrank. These are determined using the
% function pnformT.m for the given order of approximation q (recommended value q=6).
% Again, these need only be formed once for the subsequent use of Tintspade with
% various values of h.

% disp('Release version of Tintspade')
[m,n]=size(cholG);
if h==0
    TransMatrix=eye(n);
    cholCov=zeros(1,n);
    return
end
t=max(0,ceil(log2(h*scale))); % for divide and square
r=h*scale/2^t; % for scaling powers
rpow=r.^[1:q];
sr=sqrt(r/scale);
%
U=zeros(n,n);
V=U;
% form Pade approximation numerator and denominator
for k=q:-1:1
    U=-U+(pcoef(k)*rpow(k))*TscPow(:,:,k);
    V=V+(pcoef(k)*rpow(k))*TscPow(:,:,k);
end
U=eye(n)-U;
V=eye(n)+V;
TransMatrix = U\V;  % expm(A)
%
subm=[1:m];
P=zeros(qrank*m,n);
Pz=zeros(m,n);
parity=rem(q,2);
upp=q-1+parity;
par=(-1)^parity;
% form approximation to square root variance
for j=1:qrank
    Pm=Pz;
    for k=upp:-2:j
        if k>1
            Pm=Pm+(qfactor(j,k)*rpow(k-1))*GTtscPow(:,:,k-1);
        else
            Pm=Pm+qfactor(j,k)*cholG;
        end
    end
    P(subm,:)=Pm;
    subm=subm+m;
    upp=upp+par;
    par=-par;
end
[null,cholCov]=qr(P,0);
cholCov=sr*cholCov/U';
%%  Squaring
for i = 1:t
    [null, cholCov] = qr( [ cholCov; cholCov*TransMatrix'], 0);
    TransMatrix = TransMatrix*TransMatrix;
    if sum(sum(abs(TransMatrix))) < 1.e-20
        break
    end
end
