clear
clc
% A check for consistency between the state transition integrated quantities 
% calculated by alternative methods for a VCZAR model
disp('Release version of script TransitionTest')
% specify a natural VCZAR model
m=3
p=4
kappa=1.8
Phi(:,:,1) = ...
    [ -1.0937    1.8365    0.2624
    -1.5393    2.2129    0.3181
    -1.3995    1.5285    0.8901];


Phi(:,:,2) = ...
    [ -0.5793   -0.4081    0.9308
    -0.7487   -0.2700    0.9699
    -0.7499   -0.3556    1.0501];


Phi(:,:,3) = ...
    [ -1.1644    1.1057    0.1108
    -1.0450    0.9715    0.1190
    -0.8620    0.6769    0.2105];


Phi(:,:,4) = ...
    [ 1.7482   -0.1203   -1.6041
    1.5339   -0.0119   -1.4999
    1.5540   -0.0770   -1.4593];

Phi;

SigPhi = ...
    [  1.1100    1.0788    1.1390
    1.0788    1.0536    1.1113
    1.1390    1.1113    1.1918];


% transform to predictive form of model
[Xi,SigXi,M] = VCZARnatural2predictive(Phi,SigPhi, m, p, kappa); 

% Define the transition matrix predictive form:
dim = p*m; % state dimension
% Left factor
L = eye(dim) + [zeros(m,dim); [eye(dim-m), zeros(dim-m,m)]];
% Right factor
R = [reshape(Xi,m,m*p); kappa*([eye(dim-m), zeros(dim-m,m)]-[zeros(dim-m,m), eye(dim-m)])];
T=L\R; % state transition matrix
%% form right factor of state space disturbance covariance
R = chol(SigPhi); 
% R(2,:)=0;  % to test for resilience to singularity
Gt = zeros(m,dim);
Gt(1:m, 1:m)=R;
Gs = Gt*inv(L)'; % right factor of state space disturbance covariance

% carry out computations and timing ********************************************************************
rep=1000 % replication number for more accurate timings
h=1.5 % time span for transition
%
%
disp('timing for eigenvalue method for factor')
% eigen analysis for use by Tintseig
[Vem,Eem,dem,GsVit,stab,eqcrit]=eigsort(T,Gs);
tic; 
for i=1:rep
    [TdiscA, cholTA] = Tintseig(Vem,Eem,dem, GsVit, h); % values from Tintseig
end
toceig=toc
%
%
disp('timing for eigenvalue method for variance')
V=Vem;Vinv=inv(V);eps=0.00001; 
Q=GsVit'*GsVit; % variance of transformed state disturbance
tic;
for i=1:rep
    Eh=Eem*h;
    edh=exp(dem*h);
    E1=(edh*edh'-1)./Eh; % integral multiplier directly computed
    E2=1./(1-Eh/2+Eh.^2/12); % Pade approximation for small arguments to improve numerical conditioning
    E3=(abs(Eh)>eps);
    E4=E1.*E3+E2.*(1-E3); % selecting of direct or small argument elements
    QQ=Q.*E4*h; % apply integral multiplier
    VarD=real(V*QQ*V'); % transform back to integral for original states
    cholTD=chol(VarD); % factorize
end
toceigvar=toc
%
%
disp('timing for Pade method')
% quantities for use by Tintspade
qpade=6;
[pcoef,qfactor,qrank]=pnformT(qpade);
[TscPow,GTtscPow,scale] = powscalT(T,Gs,qpade);
tic;  
for i=1:rep
    [TdiscB, cholTB] = Tintspade(TscPow,GTtscPow,scale,Gs,h,qpade,pcoef,qfactor,qrank); % values from Tintspade
end
tocpade=toc
%
%
disp('timing for method by exponential of partioned matrix')
sub1=[1:dim];
sub2=dim+sub1;
tic; 
T2dim=[-T,(Gs'*Gs);zeros(dim),T'];
for i=1:rep
    T2dimh=h*T2dim;
    TdiscM2 = expm(T2dimh);
    TdiscC=TdiscM2(sub2,sub2)';
    VarC=TdiscC*TdiscM2(sub1,sub2);
    [cholTC,pp]=chol(VarC);
    if pp>0
        cholTC=zeros(dim);
    end
end
tocExpPart=toc

disp('Comparison of results from different methods of finite step state transition integration')

disp('M for built in matrix exponential of transition, as an accurate base-line')
disp('A for eigenvalue method for variance factor')
disp('B for Pade method for variance factor')
disp('C for method by built in exponential of partioned matrix')
disp('D for eigenvalue method for variance as accurate baseline for variance in this example')


disp('Comparisons of integrated transition matrix')
exphT=expm(h*T); % used to compare calculations of integrated transition matrix
checkAM=max(max(max(abs(TdiscA-exphT))))
checkBM=max(max(max(abs(TdiscB-exphT))))
checkCM=max(max(max(abs(TdiscC-exphT))))

% ensure convential signage for factored integrated variances
cholTA=diag(sign(diag(cholTA)))*cholTA;
cholTB=diag(sign(diag(cholTB)))*cholTB;
cholTC=diag(sign(diag(cholTC)))*cholTC;
cholTD=diag(sign(diag(cholTD)))*cholTD;

disp('comparison of integrated variances')
checkcholvarAB= max(max(max(abs(cholTA'*cholTA-cholTB'*cholTB))))
checkcholvarAD= max(max(max(abs(cholTA'*cholTA-VarD))))
checkcholvarBD= max(max(max(abs(cholTB'*cholTB-VarD))))
checkcholvarCD= max(max(max(abs(cholTC'*cholTC-VarD))))

disp('comparison of factored integrated variances')
checkcholAB=max(max(max(abs(cholTA-cholTB))))
checkcholAD=max(max(max(abs(cholTA-cholTD))))
checkcholBD=max(max(max(abs(cholTB-cholTD))))
checkcholCD=max(max(max(abs(cholTC-cholTD))))

