function [irf,srf,lvec]=VCZARIrf(Phi,SigPhi,kappa,laglim,lagnum,feedback)
% Generates the impulse and step response of an output series to an input series
% for a bivariate natural VCZAR model with coefficients Phi, SigPhi and parameter
% kappa at the number lagnum of distinct lags up to a specified lag limit laglim.
% A state formulation is used with an input of impulse form, and Matlab expm
% function for the transition step. The first series is the input,
% the second the output.

% disp('Release version of function VCZARIrf')
[m,null,p]=size(Phi);
if m~=2
    disp('Must be bivariate system')
    return
end
h=laglim/lagnum;
lvec=h*[0:lagnum];
% transform to predictive form to set up state transition
[Xi,SigXi] = VCZARnatural2predictive(Phi,SigPhi, m, p, kappa);
%
areg=SigXi(2,1)/SigXi(1,1) ; % contemporaneous regression of output on input used to initialise SRF
% Zero the dynamics of X if no feedback desired
if feedback==0
    Xi(1,:,:)=0.0; % remove coefficents of dependence of series 1 on series 2
end
SigXi=zeros(m); % for noise free response
%% set up state structures
dim = p*m; % state dimension
subm=[1:m];
% Define the transition matrix predictive form:
% Left factor
L = eye(dim) + [zeros(m,dim); [eye(dim-m), zeros(dim-m,m)]];
% Right factor
R = [reshape(Xi,m,m*p); kappa*([eye(dim-m), zeros(dim-m,m)]-[zeros(dim-m,m), eye(dim-m)])];
T=L\R; % state transition matrix
Gs = zeros(m,dim); % do not need variances
% Now sort for input and output states
suby=2*[1:p];
subx=suby-1;
Tyy=T(suby,suby);
Tyx=T(suby,subx);
%
% FIX for stability of IRF
if feedback==0
    epert=0.0;
    check=eig(Tyy);
    em=max(real(check));
    if em>0
        mdec=0.00001
        epert=em+mdec*kappa;
        disp('Stability correction by:')
        epert
        Tyy=Tyy-epert*eye(p);
    end
    T(suby,suby)=Tyy;
end
% eigen analysis for possible use in updating
stab=0;
[Vem,Eem,dem,GsVit,stab,eqcrit]=eigsort(T,Gs);
%% Check for stability of the model
if stab==1 % model unstable
    disp('model unstable')
    return
end
% Initialise the step response states
Svec=zeros(2*p,1);
StatImp=-(-1).^[1:p]';
aStatImp=areg*StatImp;
Svec(subx)=StatImp;
Svec(suby)=0;
Svec(suby)=aStatImp; % contemporaneous dependence of output on input
sel=2;
for k=0:lagnum
    M=expm(T*h*k);
    Statirf=M(suby,suby)*aStatImp + M(suby,subx)*StatImp;
    srf(k+1)=Statirf(1); % extract step response
    if k>0
        irf(k+1)=(srf(k+1)-srf(k))/h; % difference for impulse response
    else
        irf(k+1)=srf(k+1)/h;
    end
end
return



