function results=...
    VCZARestimate_reml(y,p,kappa,parind,obsnoise,xreg,struc,initpar,constraints,controlpar,strategy)
% Maximum restricted likelihood estimation of Vector Continuous time ZAR model 
% by state space computations with regressors included as states
% 
% INPUTS:
% y       := continuous m-vector time series dataset
%          this is a structure of m cells, each successive cell containing
%          a continuous time series composed of two columns:
%          the observation time, and the value observed at that time.
% p       := CZAR model order
% kappa   := CZAR model decay rate
% parind  : parameterisation indicator: 1 for natural, 2 for predictive
% obsnoise:= one for diagonal form, two for full, zero to omit
% xreg    := explanatory regression variables
%          this is a cell structure, each successive cell corresponding  
%          to those of the time series, and containing a regression matrix
%          of explanatory variables for that time series, at the same
%          observation times. Set {} if none.
% struc   := zero to omit, one for structural error, two for full structural model
% initpar := values to initialise non-linear model parameters. Include only the
%         parameter groups which are to be estimated, giving a structure of two, 
%         three or four cells described below and in that order: if set to {} 
%         or any cell set to [], the following defaults are used as appropriate:
%         VCZARcoeffinit : inital values of VCZAR coefficient array 
%                          if [], set to zeros(m,m,p) (parameter group I)
%         VCZARSiginit : initial values of disturbance variance matrix of the model
%                        if struc>0 all off-diagonals are reset to zero
%                        if [], set to a diagonal function of y (parameter group II)
%         NoiseSiginit : initial values of the vector of the diagonal observation noise
%                        variance matrix if obsnoise==1 or the full variance matrix if
%                        obsnoise==2; if [], set to zero vector or matrix. 
%                        Omit if obsnoise==0. (parameter group III)
%         Strucoeffinit: initial values of the structural model matrix. Diagonals are
%                        reset to ones. If [] set to unit diagonal matrix. 
%                        Omit if struc==0. (parameter group IV) 
% constraints  := cell structure comprising a parameter constraint vector
%                 for each group of model parameters I, II, III, IV;
%                 value of one to estimate, of zero to retain initial setting. 
%                 The vector applies to matrix elements row by row 
%                 (last index varying most rapidly), but only 
%                 to the upper half of a general variance matrix, to the
%                 diagonal elements of a structural disturbance variance
%                 matrix, and to the non-diagonal elements of the
%                 structural dependence matrix.
%                 If set to {} or any cell set to [] the default interpretation is used
%                 as appropriate. That is: all parameters are estimated except that
%                 if struc>0 the default is that only the structural model matrix upper 
%                 triangular elements are estimated and the diagonals of the 
%                 disturbance variance matrix.
%                                 
% controlpar  := search control parameter vector with elements as below;
%          if truncated, defaults are set:
%          maxiter  : the maximum number of iterations
%          convcrit : the convergence criterion for deviance reduction,
%          monind   : monitoring indicator:
%                   : 0 for no monitoring
%                   : 1 for dev at main iteration
%                   : 2 for dev at each iteration
%                   : 3 for dev and lam 
%          facup    : for search control - increase in constraint lam,
%          facdn    : for search control - decrease in constraint lam,
%          laminit  : for search control - initial value of lam
%          lamlow   : for search control - lower limit on lam,
%          lammax   : for search control - upper limit on lam,
%          delta    : parameter perturbation for numerical derivatives
%          maxinn   : maximum for inner loop of search,
%          eps      : singularity criterion.
% strategy: matrix specifying an initial loop pattern of strategic search
%           and a continuing loop pattern of strategic search.
%           each row consists of four 0 or 1 indicators followed by a positive integer (r)
%           the indicators specify which of the parameter groups (as defined for initpar above)
%           are to be estimated in that iterative substep which is repeated r times
%           a zero row separates the initial from the continuing pattern
%           the initial pattern is used once to start the estimation process
%           the continuing pattern is repeated within each subsequent main step of the iterations
%           If set to {} the default used is 
%         [  1, 0, 0,  1, 2    % autoregressive and structural coefficients
%            0, 1, 1,  0, 2    % disturbance and observation noise terms
%            0, 0, 0,  0, 0    % separation row
%            1, 1, 1,  1, 1    % all parameters simultaneously
%            1, 0, 0,  1, 1    % autoregressive and structural coefficients
%            0, 1, 1,  0, 1 ]; %disturbance and observation noise terms
%           The matrix is internally amended to incorporate parameter constraints and options
% -------------------------------------------------------------------------
%
% OUTPUTS: results is a cell structure with the following elements
%
%  1: VCZARcoeff   := estimated values of CZAR coefficient array
%  2: VCZARSig     := estimated variance matrix of the VCZAR model
%  3: NoiseSig     := estimated observation noise variance, diagonal or full matrix
%  4: xregcoeff    := estimated regression coefficients as vector
%  5: xregvarmx    := conditional variance matrix of regression coefficients
%  6: Strucoeff    := estimated values of structural model matrix
%  7: parstderr    := standard error array of the ordered parameter estimates excluding regression 
%  8: parcorrmx    := correlation matrix of the ordered parameter estimates excluding regression
%  9: dev          := deviance of the fitted model; refer differences in dev to chi-squared
% 10: pdev         := psuedo-deviance analagous to residual SS (AIC is N*log(pdev)+2*m^2*p) 
% 11: df           := residual degrees of freedom
% 12: stanperr     := vector of standardized prediction errors
% 13: predsdev     := vector of standard errors of predictions
% 14: yRef         := structured multivariate time series data
%               first  column = series number
%               second column = original index number of that series
%               third column  = time value of that series
%               fourth column = series value
% 15: totmodpar    := total number of model parameters
% 16: subindex     := an index of locations of estimated parameters excluding regression
% 17: xregtot      := total number of regressors
% 18: FailInd      := vector of indicators of failure conditions
%                Failind(1)=1: equations for new estimates singular
%                Failind(2)=1; VCZARsig non-positive if struc==0; negative if struc>0
%                Failind(3)=1; NoiseSig negative or non-positive resp obsnoise==1,2 
%                Failind(4)=1; lack of model stability
%                Failind(5)=1; Deviance non-reduction
%                Failind(6)=1; Failure to find Deviance reduction
% -------------------------------------------------------------------------

%% set control parameters
%  set defaults

disp('Release version of VCZARestimate_reml')

FailInd=[0,0,0,0,0,0]; % initialise

maxiter =50; 
convcrit=0.0001; 
monind  = 2; % (for development should be 1)
facup   = 5.0; 
facdn   = 5.0; 
laminit = 0.01;
lamlow  = 0.0001; 
lammax  = 10000;
delta   = 1.e-7;
maxinn  = 20; 
eps     = 1.e-9; 
defcontrolpar=[maxiter,convcrit,monind,facup,facdn,laminit,lamlow,lammax,delta,maxinn,eps];
ncp=length(controlpar);
%  over-ride defaults
if ncp>0
    defcontrolpar(1:ncp)=controlpar;
end
maxiter =defcontrolpar(1);
convcrit=defcontrolpar(2);
monind  =defcontrolpar(3); 
% facup   =defcontrolpar(4); % used in search.m
% facdn   =defcontrolpar(5); % used in search.m
laminit =defcontrolpar(6);
% lamlow  =defcontrolpar(7); % used in search.m
% lammax  =defcontrolpar(8); % used in search.m
delta   =defcontrolpar(9);
% maxinn  =defcontrolpar(10); % used in search.m
% eps     =defcontrolpar(11); % used in search.m

%% get overall series length and sequence indicators
m = length(y);
yRef=[];
for i=1:m
    yi = y{i};
    [n, is2] = size(yi);
    yiRef = i*ones(n,1);
    yiSeq = [1:n]';
    yRef = [yRef; [yiRef,yiSeq,yi]];
end  


% create yRef with:
%     first  column = series number
%     second column = original index number of that series
%     third column  = time value of that series
%     fourth column = series value
[null,sortIndex] = sort(yRef(:,3));
yRef = yRef(sortIndex,:);
Nobs=size(yRef,1);

%% Evaluate series SDs for use later
for i=1:m
    sdy(i)=sqrt(var(y{i}(:,2)));
end
% determine numbers of regressors
xregnum=zeros(1,m);
xregind=sum(size(xreg))>0;
if xregind==1
    for i=1:m
        xregnum(i)=size(xreg{i},2);
    end
end    
xregtot=sum(xregnum);


%% assign intial parameter values 
% should really check that all initial parameters set by user have valid size and values

lip=length(initpar);

VCZARcoeffinit=[];
VCZARSiginit=[];
NoiseSiginit=[];
xregcoeffinit=[];
Strucoeffinit=[];

gpind=zeros(1,5); % index of parameter group allowing for omitted groups
gpind(1)=1;
gpind(2)=2;
gpct=2;
if obsnoise==0
    gpind(3)=0;
else
    gpct=gpct+1;
    gpind(3)=gpct;
end
if struc==0
    gpind(4)=0;
else
    gpct=gpct+1;
    gpind(4)=gpct;
end

if lip>0 % assign user supplied initial values to parameter groups
    if sum(size(initpar{1}))>0
        VCZARcoeffinit=initpar{1};
    end
    if sum(size(initpar{2}))>0
        VCZARSiginit=initpar{2};
        if struc>0
            VCZARSiginit=diag(diag(initpar{2})); % reset off-diagonals to zero 
        end
    end
    k=gpind(3);
    if k>0
        if sum(size(initpar{k}))>0
            NoiseSiginit=initpar{k};
        end
    end
    k=gpind(4);
    if k>0
        if sum(size(initpar{k}))>0
            Strucoeffinit=initpar{k};
        end
    end
end

% assign default parameter values as required

npar=zeros(1,4); % to hold number of potential parameters ignoring constraints

npar(1)=m*m*p; % number of parameters of type I
if sum(size(VCZARcoeffinit))==0
    VCZARcoeffinit=zeros(m,m,p);
    if parind==2
        VCZARcoeffinit(:,:,1)=-kappa*eye(m);
    end
end

if struc==0
    npar(2)=m*(m+1)/2; % number of parameters of type II
else
    npar(2)=m; % diagonal values only
end
if sum(size(VCZARSiginit))==0
    VCZARSiginit=zeros(m,m);
    for i=1:m
        VCZARSiginit(i,i)=sdy(i)^2;
    end
end

npar(3)=0; % number of parameters of type III
if obsnoise==1
    if sum(size(NoiseSiginit))==0
        NoiseSiginit=0.01*diag(VCZARSiginit);
    end
    npar(3)=m;
elseif obsnoise==2
    if sum(size(NoiseSiginit))==0
        NoiseSiginit=0.01*diag(diag(VCZARSiginit))
    end
    npar(3)=m*(m+1)/2;
end

npar(4)=0; % number of parameters of type V
if struc>0
    if sum(size(Strucoeffinit))==0
        Strucoeffinit=eye(m);
    end
    npar(4)=m^2-m; % potentially all off-diagonals
end


%% set default constraints
if sum(size(constraints))==0
    constraints={[],[],[],[]};
end
if sum(size(constraints{1}))==0
    constraints{1}=ones(npar(1),1);
end
if sum(size(constraints{2}))==0
    constraints{2}=ones(npar(2),1);
end
if sum(size(constraints{3}))==0
    if obsnoise>0
        constraints{3}=ones(npar(3),1);
    else
        constraints{3}=[];
    end
end
if sum(size(constraints{4}))==0
    if struc>0 % specify estimation of upper triangular elements only of off-diagonals
        concol=zeros(m,1);
        concol(1)=1;
        conrow=ones(1,m-1);
        conmat=toeplitz(concol,conrow);
        constraints{4}=reshape(conmat',1,m^2-m); 
    else
        constraints{4}=[];
    end
end
%% count constraints
for g=1:4
    nactivpar(g)=sum(constraints{g});
end
totactivpar=sum(nactivpar);
% celldisp(constraints)
% totactivpar

if totactivpar==0
    display('Only regressors to estimate')
    %     return
end

%% set search strategy
if sum(size(strategy))==0
    %     defstrategy=...
    %       [ 1, 1, 1,  1, 1
    %         1, 0, 0,  1, 3
    %         0, 1, 0,  0, 1
    %         0, 1, 1,  0, 3
    %         0, 0, 0,  0, 0
    %         1, 1, 1,  1, 1
    %         0, 1, 1,  0, 3 ];
    defstrategy=...  % revised Feb 2012 - improved startup from defaults
        [ 1, 0, 0,  1, 2
         0, 1, 1,  0, 2
         0, 0, 0,  0, 0
         1, 1, 1,  1, 1
         1, 0, 0,  1, 1
         0, 1, 1,  0, 1 ];
    
else
    defstrategy=strategy;
end
% strategy
% final strategy for covariances
finstrategy=[1,1,1,1,1];
% introduce effects of options and constraints
for g=1:4
    if nactivpar(g)==0
        defstrategy(:,g)=0;
        finstrategy(g)=0;
    end
end
minrowcycle=find(defstrategy(:,5)==0);
% distill out rows that are all zeros
rsav=find(sum(defstrategy(:,1:4),2)>0);
dstrategy=defstrategy(rsav,:);
maxrowcycle=size(dstrategy,1);
if maxrowcycle>0
    minrowcycle=min(find(rsav>minrowcycle));
else
    dstrategy=finstrategy;
    maxrowcycle=1;
    minrowcycle=1;
end
%disp('dstrategy set')
%% set initial parameter vector and perturbation scales
count=0;
totmodpar=sum(npar);
parvec=zeros(totmodpar,1);
pertscale=zeros(totmodpar,1);
for i=1:m % for VCZARcoeff note sequence of i,j,k - last index changing most rapidly (k,i,j used for discrete VZAR)
    for j=1:m
        pertsc=sdy(i)/sdy(j);
        for k=1:p
            count=count+1;
            parvec(count)=VCZARcoeffinit(i,j,k);
            pertscale(count)=pertsc;
        end
    end
end
if struc==0 % for VCZARSig
    for i=1:m 
        for j=i:m % note symmetry
            count=count+1;
            parvec(count)=VCZARSiginit(i,j);
            pertscale(count)=sdy(i)*sdy(j);
        end
    end
else
    for i=1:m
        count=count+1;
        parvec(count)=VCZARSiginit(i,i);
        pertscale(count)=sdy(i)^2;
    end
end
if obsnoise==1
    for i=1:m % for diagonal NoiseSig
        count=count+1;
        parvec(count)=NoiseSiginit(i);
        pertscale(count)=sdy(i)^2;
    end
elseif obsnoise==2
    for i=1:m % for full NoiseSig
        for j=i:m % note symmetry
            count=count+1;
            parvec(count)=NoiseSiginit(i,j);
            pertscale(count)=sdy(i)*sdy(j);
        end
    end
end

if struc>0
    for i=1:m
        for j=1:m
            if i~=j
                count=count+1;
                parvec(count)=Strucoeffinit(i,j);
                pertscale(count)=sdy(i)/sdy(j);
            end
        end
    end
end

%%%%%%*********************************************************************

%% loop through iterations
% first evaluate initial deviance
% parvec % check on initial parameters setting
% [VCZARcoeff,VCZARSig,NoiseSig,Strucoeff]=parvec2true_reml(parvec,m,p,obsnoise,struc) % check on initial parameters setting
[stanperr, predsdev, dev, stab, DSig, NSig, xregcoeff, xregvarmx] =...
    VCZARlikfun_reml(yRef,m,p,kappa,obsnoise,xreg,xregnum,parvec,parind,struc);
if stab==1
    disp('Initial values VCZARcoeffinit not stable')
    results{17} =[0,0,0,1,0,0];
    return
end

logprod=sum(log(predsdev));
pdev=sum(stanperr.^2)*exp(2*logprod/Nobs);
disp(['Initial deviance',sprintf('\t%0.10g',dev)])
disp(['Initial psueddev',sprintf('\t%0.10g',pdev)])
olddev=dev;
chkdev=dev;
oldparvec=parvec;
oldlam=laminit;
rowcycle=0;
strep=0;
limiter=5*maxiter;
mainct=0;
% for iterct = 1 : limiter
iterct=0;
% maxiter
parstderr=[];
parcorrmx=[];
subindex=[];
if totactivpar > 0
    while mainct <=maxiter
        iterct=iterct+1;
        if strep==0
            rowcycle=rowcycle+1;
            if (rowcycle>maxrowcycle)
                rowcycle=minrowcycle;
            end    
            rstrategy=dstrategy(rowcycle,:);
            strep=rstrategy(5);
            mainstep=rstrategy(1);
        end
        strep=strep-1;
        if mainstep==1
            mainct=mainct+1;
        end
        [subindex,lensubpar]=subparindex_reml(rstrategy,constraints);
        [y,e,X,Z,scFail,stab,DSig,NSig]=scoreconstruct_reml(yRef,m,p,kappa,obsnoise,xreg,xregnum,...
            subindex,lensubpar,parvec,parind,struc,pertscale,stanperr,predsdev,delta);
        if scFail==1
            Iprogress(2)=DSig>0;
            Iprogress(3)=NSig>0;
            Iprogress(4)=Stab>0;
            disp(['Estimation fails with indicators:',sprintf('\t%0.6g',Iprogress)])
            break
        end
        [newsubparvec,newstanperr,newpredsdev,newdev,Iprogress,newlam]=...
            search_reml(y,e,X,Z,subindex,lensubpar,oldparvec,parind,struc,olddev,oldlam,...
            yRef,m,p,kappa,obsnoise,xreg,xregnum,defcontrolpar);
        if sum(Iprogress)==0  %% update structures
            parvec(subindex)=newsubparvec;
            stanperr=newstanperr;
            predsdev=newpredsdev;
            dev=newdev;
            logprod=sum(log(predsdev));
            remlNobs=length(stanperr);
            pdev=sum(stanperr.^2)*exp(2*logprod/remlNobs);
            if (monind>1) | ( (monind)==1 & (mainstep==1) )
                disp(['Iteration count',sprintf('\t%0.5g',iterct),...
                        '   Mainstep count',sprintf('\t%0.5g',mainct)])
                disp(['Deviance',sprintf('\t%0.10g',dev)])
                disp(['Psueddev',sprintf('\t%0.10g',pdev)])
            end
            if (mainstep==1)&(newdev>(chkdev-convcrit))
                disp('Convergence')
                break
            else
                if (mainstep==1)
                    chkdev=dev;
                end
                olddev=dev;
                oldparvec=parvec;
                oldlam=newlam;
                %             disp('Update oldlam to newlam')
                %             oldlam
                if (monind==2) & (mainstep==1)
                    %                 disp(['lambda',sprintf('\t%0.6g',newlam)])
                end
            end
        else
            disp(['Estimation fails with indicators:',sprintf('\t%0.6g',Iprogress)])
            break
        end
    end
    % last construct to get SEs corrs.
    parstderr=[];
    parcorrmx=[];
    [stanperr, predsdev, dev, stab,  DSig, NSig, xregcoeff, xregvarmx] =...
        VCZARlikfun_reml(yRef,m,p,kappa,obsnoise,xreg,xregnum,parvec,parind,struc);
    [subindex,lensubpar]=subparindex_reml(finstrategy,constraints);
    if (stab+DSig+NSig)>0
        disp('Final model parameters invalid')
    else  
        [y,e,X,Z,scFail,stab,DSig,NSig]=scoreconstruct_reml(yRef,m,p,kappa,obsnoise,xreg,xregnum,...
            subindex,lensubpar,parvec,parind,struc,pertscale,stanperr,predsdev,delta);
        [Q,R]=qr([X;sqrt(2)*Z],0); % Used to form se and correlations of estimates
    end
    if scFail==0
        r=diag(R);
        sqrt2=sqrt(2);
        d=sqrt(sum([X;sqrt2*Z].^2));
        if min(abs(r./d'))<eps
            sing=1
        else
            sing=0
        end
        if sing==0
            Ri=inv(R);
            V=(Ri*Ri');
            dV=diag(V);
            parstderr=sqrt(dV);
            parcorrmx=V./(parstderr*parstderr');
        end
    end
    %% set output values on completion
    
    FailInd=Iprogress;
else
    [stanperr, predsdev, dev, stab, DSig, NSig, xregcoeff, xregvarmx] =...
        VCZARlikfun_reml(yRef,m,p,kappa,obsnoise,xreg,xregnum,parvec,parind,struc);
end  % of code skipped for totactivpar==0

[VCZARcoeff,VCZARSig,NoiseSig,Strucoeff]=parvec2true_reml(parvec,m,p,obsnoise,struc);
logprod=sum(log(predsdev));
remlNobs=length(stanperr);
pdev=sum(stanperr.^2)*exp(2*logprod/remlNobs);
df=remlNobs-totactivpar;

results=...
    {VCZARcoeff,VCZARSig,NoiseSig,xregcoeff,xregvarmx,Strucoeff,parstderr,parcorrmx,dev,pdev,...
        df,stanperr,predsdev,yRef,totmodpar,subindex,xregtot,FailInd};
return
