function [statfil,statsmo,trend,Vstatfil,Vstatsmo,cTranMat] =...
    VCZstateFilterSmooth(yRef,m,p,kappa,obsnoise,xreg,xregnum,...
    VCZARcoeff,VCZARSig,NoiseSig,xregcoeff,Strucoeff,parind,struc)
% Uses State Space formulation of the Continuous Vector ZAR model
% to estimate filtered and smoothed states including at unobserved time points
% these times are specified in yRef but with NaN for observations
% xreg values will also need to be given at interpolated points
% states are estimated in the time sequence given in yRef including simultaneity
% statfil and statsmo are respectively the filtered and smoothed states at these times
% trend is the correction applied to the series value of current observation
% Vstatfil and Vstatsmo are respectively their variance matrices
% observations noise is appended to the state vector
% cTranMat is the continuous time transition matrix for CZAR states
% useful for transforming state estimates to decoupled eigencomponents

disp('Release version of VCZstateFilterSmooth')
[Nobs, is3] = size(yRef);      % number of time points at which states are to be estimated
tspan=yRef(Nobs,3)-yRef(1,3); % overall time span
hamo=tspan*1.e-12; % successive times closer than this are taken to be the same.
% could introduce eqlim and qpade as control parameters
eqlim=0.01; % determines eigen method (small value) or Pade method (large) for update
eqlim=100.0;
qpade=6; % order of Pade approximations


%% map to model parameters
xregtot=sum(xregnum);
xregorigins=cumsum([0,xregnum(1:(m-1))]); % for use in locating coefficients in xregcoeff
% square root factor noise variance allowing for singularity
if obsnoise==1
    NoiseSD=sqrt(NoiseSig);
elseif obsnoise==2
    sqrtNS=sqrtm(NoiseSig);
    [null,NoiseSD]=qr(sqrtNS)
end

% map the structural parameters to standard form 

Sig=VCZARSig;
coeff=VCZARcoeff;

if struc>0
    Sig=(Strucoeff\Sig)/Strucoeff';
end
if struc==2
    for k=1:p
        coeff(:,:,k)=Strucoeff\coeff(:,:,k);
    end
end


if parind==1
    [Xi,SigXi,M] = VCZARnatural2predictive(coeff,Sig, m, p, kappa); 
else
    Xi=coeff;
    SigXi=Sig; 
end
%% set up state structures
dim = p*m; % state dimension
dimx = dim; % length of householder vector
if obsnoise>0
    dimx=dimx+ m; % augmented if observation noise
end

%% initialize output quantities
% - all variances as right factors before converting to true variances for return
trend=zeros(1,Nobs);
statfil=zeros(dimx,Nobs );
statsmo=zeros(dimx,Nobs);
Vstatfil=zeros(dimx,dimx,Nobs);
Vstatsmo=zeros(dimx,dimx,Nobs);
%% initalize further quantities to be saved for smoothing 
statpred=zeros(dimx,Nobs );
RegMat=zeros(dimx,dimx,Nobs); % transposed regression of estimated state seq-1 upon estimated state seq
RegMatvarfac=zeros(dimx,dimx,Nobs); % right factor of reduction in variance from filtered to smoothed state seq
DelMat=zeros(dimx,dimx,Nobs); % right factor of reduction in variance from filtered to smoothed state seq

% Define the transition matrix direct form:
% Left factor
L = eye(dim) + [zeros(m,dim); [eye(dim-m), zeros(dim-m,m)]];
% Right factor
R = [reshape(Xi,m,m*p); kappa*([eye(dim-m), zeros(dim-m,m)]-[zeros(dim-m,m), eye(dim-m)])];

T=L\R; % state transition matrix
cTranMat=T;
%% right factor of state space disturbance covariance
Gt = zeros(m,dim);
if struc==0
    sqrtVS = sqrtm(VCZARSig);
    [R,null] = chol(sqrtVS); 
else 
    R= diag(sqrt(diag(VCZARSig)));
    R=R/(Strucoeff');
end
if parind==1
    R=R/(M');
end
Gt(1:m, 1:m)=R;
Gs = Gt*inv(L)'; % right factor of state space disturbance covariance
% eigen analysis for possible use in updating
stab=0;
[Vem,Eem,dem,GsVit,stab,eqcrit]=eigsort(T,Gs);
%% Check for stability of the model
if stab==1 % model unstable
    disp('Model unstable')
    return
end
if eqcrit<eqlim
    eqind=1;
    [pcoef,qfactor,qrank]=pnformT(qpade);
    [TscPow,GTtscPow,scale]=powscalT(T,Gs,qpade);
else
    eqind=0;
end

%% Initialise right Choleski factor P of state covariance matrix
initsc=1.e6;
hin=(1/kappa)*initsc; % initialisation period 

% Householder vectors and matrix resultant
F=zeros(dimx,1);
Lh=zeros(dimx,1);
stsub=[1:dim]; % to address state subvectors
msub=[1:m];
if obsnoise>0
    obsub=[dim+1:dimx]; % to address obs. noise subvectors
end
astsub=[1:dimx]; % to address augmented states
mzero=zeros(m,1);
R=zeros(dimx+1,dimx);
Sest = zeros(dim,1); % initial state estimate
SestA = zeros(dimx,1); % augmented state vector
cholP = zeros(dim,dim); % state variance factor 
cholPAzero = zeros(dimx,dimx); % augmented state variance factor initalised
cholPA=cholPAzero; 
%
%
% Filtering loop ********************************************************
%
% set right factor of noise variance
if obsnoise==1
    cholN=diag(NoiseSD);
elseif obsnoise==2
    cholN=NoiseSD;
end
tpast = yRef(1,3)-hin; % remote past intitial time point
for seq = 1:Nobs
    sernum=yRef(seq,1);
    serindex=yRef(seq,2);
    tnow = yRef(seq,3);
    seriesval=yRef(seq,4);
    %% adjust for regressors
    jreg=xregnum(sernum);
    trendnow=0.0;
    if jreg>0
        for j=1:jreg
            trendnow=trendnow+xregcoeff(xregorigins(sernum)+j)*xreg{sernum}(serindex,j);
        end
    else
        trendnow=0;
    end
    %     save trend
    trend(seq)=trendnow;
    %
    % apply transition update *************************************************
    %
    %     if tnow > tpast + hamo 
    h = tnow - tpast;
    if eqind==1
        [Tdisc, cholT] = ...
            Tintspade(TscPow,GTtscPow,scale,Gs,h,qpade,pcoef,qfactor,qrank);
        %             disp('pade method')
    else 
        [Tdisc, cholT] = Tintseig(Vem,Eem,dem, GsVit, h);
        %             disp('eigen method')
    end
    if seq==1 % use stationary state to set tolerances
        eps=1.e-7; % tolerance for exactly known state variable
        tolvec=diag(cholT'*cholT);
        if obsnoise>0
            tolvec=[tolvec;tolvec(msub)]; % take states correp. to obsns for noise tolerance
        end
        tolvec=sqrt(tolvec);
        veceps=eps*tolvec; % for detecting state dependency
    end
    %****************************************************************************
    % Here to evaluate backward regression of state(seq-1) on state(seq) for use in smoother
    % 
    [lenT,null]=size(cholT); % lenT may be < dim if lower rank 
    [lenP,null]=size(cholP); % lenP may be dim or dimx at different stages
    %first determine number nid of linearly independent augmented states seq (including noise)
    % and index E of re-ordering of states by increasing dependence
     A = zeros(lenT+lenP,dimx);
     A([1:lenP],stsub)=cholP*Tdisc';
     A([1:lenT]+lenP,stsub)=cholT;
     if obsnoise>0
        A(msub+lenP+lenT,msub+dim)=cholN;
    end
    [Q,R,E]=qr(A,0);
    nid=sum(abs(diag(R))>veceps(E)); % take re-ordered states beyond first nid to be dependent

    % form B st B'*B gives joint variance of estimated state(seq-1) and predicted state(seq) 
    lenB=lenT+dimx;
    if obsnoise>0
        lenB=lenB+m;
    end
    dimx2=dimx*2;
    B=zeros(lenB,dimx2);
    B([1:lenP],stsub)=cholP*Tdisc';
    B([1:lenT]+dimx,stsub)=cholT;
    if obsnoise>0
        B(msub+dimx+lenT,obsub)=cholN;
    end
    B(astsub,dimx+astsub)=cholPA; % includes correlation of noise with states
    B(:,astsub)=B(:,E); % independent predicted states are permuted into first nid positions
    [Q,R]=qr(B,0); % R is right factor of joint variance of predicted permuted state(seq) and estimated state(seq-1)
    R(:,E)=R(:,astsub); % restore original column ordering of state(seq) 
    subid=[1:nid];
    Esort=E(subid); % column numbers of independent states(seq)
    % now determine the singular regression using submatrix of first nid rows and columns of independent states(seq)
    A=R(subid,Esort); % non-singular submatrix for independent states(seq)
    C=R(subid,dimx+astsub);
    D=A\C;
    RegMat(Esort,:,seq)=D; % transposed regression coefficient stored with submatrix set for independent state(seq) rows
    % derivation of regression error variance factor
    [b,a]=size(R);
    A=R(nid+1:b,dimx+astsub); % includes extra error contribution due to dependence in states and direct contribution
    [Q,R]=qr(A,0); 
    [a,b]=size(R);
    RegMatvarfac(1:a,1:b,seq)=R; % combined error factor stored

    %***********************************************
    % Backward regression quantities now completed

    % state prediction and variance factor formed in following
    Sest = Tdisc*Sest;    % now Augmented to full state with noise if required
    SestA(stsub)=Sest;
    statpred(:,seq)=SestA;
    if obsnoise>0
        SestA(obsub)=mzero;
    end
    
    % continue with state prediction step
    
    A = [cholP*Tdisc'; cholT];
    [null, cholP] = qr(A, 0); % get NEW cholP
    [lenPnew,null]=size(cholP);

    
    cholPA=cholPAzero;
    cholPA([1:lenPnew],stsub)=cholP;
    
    if obsnoise>0
        cholPA(obsub,obsub)=cholN;
    end
    %     
    
    % Sest will remain the same if tnow=tpast except predicted noise must be zeroed
    tpast = tnow;
    %
    % observation prediction and conditioning for state filtering % *******************************************************
    %
    if ~isnan(seriesval)
        seriesval=seriesval-trendnow;
        ypred = SestA(sernum);     % prediction of the series observed at this time
        F = cholPA(:,sernum);  % prediction factor of selected observation for Householder
        if obsnoise>0
            F = F + cholPA(:,dim+sernum);
        end
        R = cholPA;
        [Lh,cholC] = hhrv(F);          % householder reflection vector and scalar
        R = R-2*Lh*(Lh'*R);           % application of reflection to R
        K = R(1,:)';                  % Kalman gain as applied to standardised errors
        R(1,:) = 0;
        cholPA=R; % right factor of updated state variance, no longer upper triang
        cholP=cholPA(:,stsub);  % retained for use after time increment
        % cholP first dimension now extended but right factor property retained
        signC = 2*(cholC>0)-1;
        prederr = seriesval-ypred ;    % prediction error
        serr = prederr/cholC; 
        stanperr(seq) = serr*signC;   % standardized error series
        predsdev(seq) = cholC*signC;
        SestA = SestA + K*serr;         % updated state estimates - if noise, estimates may be non-zero
        Sest = SestA(stsub); % retained for use after time increment
        Kgaint(:,seq)=K;
    end
    % if series value missing SestA and cholPA, cholP unchanged from prediction
    statfil(:,seq)=SestA;  % save estimated current augmented state
    Vstatfil(:,:,seq)= cholPA; % save variance factor of estimated augmented state
    % square with first row of zeros 
end

% backward smoothing loop

statsmo(:,Nobs)=statfil(:,Nobs);
Vstatsmo(:,:,Nobs)=Vstatfil(:,:,Nobs);
for seq=Nobs-1:-1:1
    statsmo(:,seq)=statfil(:,seq)+RegMat(:,:,seq+1)'*(statsmo(:,seq+1)-statpred(:,seq+1));% smoothed state estimate
    B=Vstatsmo(:,:,seq+1);
    C=RegMat(:,:,seq+1);
    R=RegMatvarfac(:,:,seq+1);
    A=[R;B*C]; % compounded right factor regression error, both direct and prediction from state(seq)
    [Q,R]=qr(A,0); 
    Vstatsmo(:,:,seq)=R; % right factor smoothed state estimate variance
end
eps=0.00; % non-zero for comparison plot check with previous algorithm
for seq=1:Nobs
    P=Vstatfil(:,:,seq);
    Vstatfil(:,:,seq)=P'*P+eps;
    P=Vstatsmo(:,:,seq);
    Vstatsmo(:,:,seq)=P'*P+eps;
end







